/*

2645_FSM_UpDown_Counter_Transition_Table

Sample code from ELEC2645

Demonstrates how to implement a simple FSM up/down counter using a transition table

(c) Craig A. Evans, University of Leeds, Jan 2016
Updated Jan 2020

*/

#include "mbed.h"

// pre-processor directives

// defines directions as 0/1. Note UPPERCASE
#define UP 0
#define DOWN 1

// K64F on-board LEDs
BusOut k64f_leds(LED_RED, LED_GREEN, LED_BLUE);

// Gamepad Button A
InterruptIn buttonA(PTC7);

// LEDs on Gamepad (1 to 3) - active-low 0 = on and 1 = off
// BusOut arguments are LSB first
BusOut output(PTA2,PTC2,PTC3);

// struct for state
struct State {
    int output;  // output value for current state
    float time;  // wait time for state
    int next_state[2]; // next state (depending on direction 0 - UP, 1 - DOWN)
};

// array of states in the FSM, each element is the output of the counter, wait time, next state(s)
State fsm[4] = {
  {0b110,0.5,{1,2}},
  {0b101,0.5,{2,0}},
  {0b011,0.5,{0,1}},
};

// flag - must be volatile as changes within ISR
// g_ prefix makes it easier to distinguish it as global
volatile int g_buttonA_flag = 0;

// function prototypes
// Button A interrupt service routine
void buttonA_isr();

int main()
{
    k64f_leds = 0b111;  // turn off K64F LEDs
    
    // Button A is connected between the pin and 3.3 V, we therefore need to turn on the internal pull-down resister
    buttonA.mode(PullUp);
    // Setup interrupt on falling edge
    buttonA.fall(&buttonA_isr);
    
    // set initial state
    int state = 0;
    // set initial direction
    int direction = UP;

    while(1) {  // loop forever
    
        // check if flag i.e. interrupt has occured
        if (g_buttonA_flag) {
            g_buttonA_flag = 0;  // if it has, clear the flag
            // swap direction when button has been pressed
            direction = ! direction;
        }
    
        // set output of current state (use dot syntax to access struct members)
        output = fsm[state].output;  
        // implement required delay
        wait(fsm[state].time);
        // set the next state depending on direction
        state = fsm[state].next_state[direction];
        
    }
}

// Button A event-triggered interrupt
void buttonA_isr()
{
    g_buttonA_flag = 1;   // set flag in ISR
}